/*
 * #%L
 * Over-the-air deployment library
 * %%
 * Copyright (C) 2012 SAP AG
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
package com.sap.prd.mobile.ios.ota.lib;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

import com.sap.prd.mobile.ios.ota.lib.VelocityBase.IParameters;

/**
 * Base class for the Velocity based Generators Each generator has to provide an IParameters
 * implementation to provide the parameters used in the template.
 * 
 * @param <P>
 */
public abstract class VelocityBase<P extends IParameters>
{

  protected Template template;

  protected VelocityBase(String templateName)
  {
    Properties props = new Properties();
    props.put("resource.loader", "class,jar");
    props.put("class.resource.loader.class", "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");
    props.put("jar.resource.loader.class", "org.apache.velocity.runtime.resource.loader.JarResourceLoader");
    Velocity.init(props);//"velocity.props");
    template = Velocity.getTemplate(templateName);
  }

  public String generate(P parameters) throws IOException
  {
    StringWriter swriter = new StringWriter();
    PrintWriter writer = new PrintWriter(swriter);
    generate(writer, parameters);
    writer.flush();
    writer.close();
    return swriter.getBuffer().toString();    
  }

  public synchronized void generate(PrintWriter writer,
        P parameters)
        throws IOException
  {
    VelocityContext context = new VelocityContext();
    Map<String, Object> mappings = parameters.getMappings();
    for (String key : mappings.keySet()) {
      context.put(key, mappings.get(key));
    }
    template.merge(context, writer);
  }

  static interface IParameters
  {
    /**
     * key/value pairs to be put into the <code>VelocityContext</code>.
     * 
     * @return
     */
    public Map<String, Object> getMappings();
  }

  static class Parameters implements IParameters
  {
    protected final Map<String, Object> mappings;

    protected Parameters()
    {
      this.mappings = new HashMap<String, Object>();
    }

    @Override
    public Map<String, Object> getMappings()
    {
      return mappings;
    }

  }

}
